#!/usr/bin/perl -w
# 
# sb::Data::Object - Module for Serene Bach
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

package sb::Data::Object;

use strict;

# ==================================================
# // module version
# ==================================================
use vars qw( $VERSION @ISA );
$VERSION = '0.01';
# 0.01 [2005/08/23] changed alloc to be fast
# 0.00 [2005/02/02] generated

# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Text ();
# ==================================================
# // declaration for constant value
# ==================================================
sub TRUE  (){ 1 }
sub FALSE (){ 0 }
# ==================================================
# // constructor
# ==================================================
sub alloc {
	my $class = shift;
	my $self  = {
		'content_list' => ['id'],
		'trash_can' => FALSE,
		@_
	};
	bless($self,$class);
	no strict 'refs';
	my $check = $self->{'content_list'}->[1];
	# create accessor functions when the first object is allocated.
	if ( $check ne '' and !defined(&{$class . '::' . $check}) ) {
		foreach my $member ( @{$self->{'content_list'}} ) {
			next if ($member eq 'id' or $member eq 'url' or $member eq 'mail'); # reserved
			my $closure = sub {
				my $self = shift;
				$self->{$member} = shift if @_;
				$self->{$member};
			};
			*{$class . '::' . $member} = $closure unless defined(&{$class . '::' . $member});
		}
	}
	return($self);
}
sub add_new {
	my $self  = shift;
	my %param = @_;
	$self->{'id'} = $param{'id'};
	$self->initialize(%param);
	return($self);
}
# ==================================================
# // public functions - common method
# ==================================================
sub existed {
	my $self = shift;
	return( $self->{'id'} ne '' );
}
sub elements {
	my $self = shift;
	return( @{$self->{'content_list'}} );
}
sub erase {
	my $self = shift;
	return if ( !$self->existed );
	foreach my $elem ($self->elements) {
		next if ($elem eq 'id');
		$self->{$elem} = undef;
	}
	$self->{'trash_can'} = TRUE;
}
sub erased {
	my $self = shift;
	return( $self->{'trash_can'} );
}
sub initialize {
	my $self = shift;
	my %param = @_;
	foreach my $key ( keys(%param) ) {
		next if ( $key eq 'id' );
		next if ( !grep($key,@{$self->{'content_list'}}) );
		$self->{$key} = $param{$key};
	}
	return();
}
# ==================================================
# // public functions - accessor
# ==================================================
sub id { # read only
	my $self = shift;
	return( $self->{'id'} );
}
sub url {
	my $self = shift;
	$self->{'url'} = shift if (@_);
	return &_check_url($self->{'url'}) ? $self->{'url'} : undef;
}
sub mail {
	my $self = shift;
	$self->{'mail'} = shift if (@_);
	return &_check_mail($self->{'mail'}) ? $self->{'mail'} : undef;
}
sub entitize {
	my $self = shift;
	my $key  = shift;
	return( undef ) if (!$key);
	return sb::Text->entitize($self->{$key});
}
# ==================================================
# // private functions
# ==================================================
sub _check_url {
	return( $_[0] =~ /s?https?:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:\@&=+\$,%#]+/ );
}
sub _check_mail {
	return( $_[0] =~ /[\w=+\$%*-]+\@[^\s()\[\]{}!\"\'<>:,\x7f-\xff]+\.\w+/ );
}
1;
__END__
